import ldap3
import hashlib
from ldap3.utils.conv import escape_filter_chars
import ssl

def hash_from_mifare_id(_hash, pin):
    return hashlib.sha256(
        "{:08x}:{:08x}".format(
            pin,
            int.from_bytes(bytes.fromhex(_hash)[0:4], 'little')
         ).encode('ascii')
    ).digest()


class HsLdap:
    class Exception(Exception): pass

    def __init__(self, password):
        self.server = ldap3.Server('ldap.hackerspace.pl', use_ssl=True)
        self._password = password

    def _connection(self):
        return ldap3.Connection(self.server,
            'cn=access_control,ou=Services,dc=hackerspace,dc=pl',
            self._password)

    def get_hashes(self, status='active', with_neighbours=False):
        results = {}
        with self._connection() as conn:
            if status == 'active':
                filterstr = (
                    "(&"
                        "(objectClass=hsMember)"
                        "(|(memberOf=cn=starving,ou=Group,dc=hackerspace,dc=pl)"
                        "(memberOf=cn=fatty,ou=Group,dc=hackerspace,dc=pl)"
                        "(memberOf=cn=potato,ou=Group,dc=hackerspace,dc=pl))"
                    ")")
            else:
                filterstr = ("(objectClass=hsMember)")

            conn.search('ou=People,dc=hackerspace,dc=pl',
                filterstr,
                search_scope = ldap3.LEVEL,
                attributes = ['mifareIDHash', 'mifareID', 'uid'])

            for e in conn.entries:
                results[e['uid'].value] = {
                    'mifareIDHash':e['mifareIDHash'].values,
                    'mifareID':e['mifareID'].values
                }

            if with_neighbours:
                filterstr = "(objectClass=buildingMember)"
                conn.search('ou=Wolność,dc=hackerspace,dc=pl',
                    filterstr,
                    search_scope = ldap3.LEVEL,
                    attributes = ['mifareIDHash', 'mifareID', 'description'])

                for e in conn.entries:
                    results['building_' + e['description'].value] = {
                        'mifareIDHash':e['mifareIDHash'].values,
                        'mifareID':e['mifareID'].values
                    }

        return results

    def _modify_hash(self, op, uid, hashes):
        with self._connection() as conn:
            res = conn.modify(
                'uid={uid:s},ou=People,dc=hackerspace,dc=pl'.format(
                    uid = escape_filter_chars(uid)),
                { 'mifareIDHash': (op, hashes) }
            )
            if not res:
                raise self.Exception('modify hash result: {:s}'.format(
                    repr(conn.result)))

    def hash_add(self, uid, _hash, replace = False):
        if replace:
            self._modify_hash(ldap3.MODIFY_REPLACE, uid, [_hash])
        else:
            self._modify_hash(ldap3.MODIFY_ADD, uid, [_hash])

    def hash_remove(self, uid, _hash):
        assert _hash
        if uid != None:
            self._modify_hash(ldap3.MODIFY_DELETE, uid ,[_hash])
            return [uid]

        # no uid supplied - checking all members
        with self._connection() as conn:
            filterstr = (
                "(|"
                    "(objectClass=hsMember)"
                    "(objectClass=buildingMember)"
                ")")
            conn.search('ou=People,dc=hackerspace,dc=pl',
                filterstr,
                search_scope = ldap3.LEVEL,
                attributes = ['mifareIDHash', 'uid'])

            found_uids = []
            for person in conn.entries:
                if _hash in person['mifareIDHash'].values:
                    found_uids.append(person['uid'].value)

            # TODO remove hashes
            return found_uids
    

if __name__ == "__main__":
    from getpass import getpass
    hs_ldap = HsLdap(password=getpass())
    print(hs_ldap.get_hashes())
