local kube = import "../../../../kube/hscloud.libsonnet";

/*
deploy instructions:
- create a ssh keypair for git
- add the public key to https://github.com/radex/REPONAME/settings/keys
- create secret:
    kubectl -n NAMESPACE create secret generic SECRETNAME \
        --from-file=path/to/id_ed25519 \
        --from-file=path/to/id_ed25519.pub

updating data:
- set cfg.enableManagement to true, then you can run:
    kubectl -n NAMESPACE exec deploy/NAME -it -c management -- /update
- otherwise, take advantage of update on init container by restarting the pod:
    kubectl -n NAMESPACE rollout restart deploy/NAME

quirks&todo:
- use init container for initial data fetch?
- use Jobs for data updates?
- subpath in nginx container doesn't update automatically if the subpath doesn't exist,
  need to restart the pod to fix it
- nginx config could use some work
*/
{
    local top = self,
    local cfg = self.cfg,

    cfg:: {
        name: error "'name' is required",
        namespace: error "'namespace' is required",
        domains: error "'domains' is required",
        source: {
            repo: error "'source.repo' is required",
            staticSubpath: error "'source.staticSubpath' is required",
            sshSecretName: error "'source.sshSecretName' is required",
        },
        storage: error "'storage' is required",
        enableManagement: false,
        replicas: 1,
        images: {
            nginx: 'nginxinc/nginx-unprivileged:stable-alpine',
            management: 'bitnami/git:latest',
        },
        storageClassName: 'waw-hdd-redundant-3',
    },

    local ns = kube.Namespace(cfg.namespace),

    local volumes = {
        data: top.data.volume,
        config: top.config.volume,
        ssh: { secret: { secretName: cfg.source.sshSecretName } },
    },

    local makeManagementContainer(name, commands) = kube.Container(name) {
        image: cfg.images.management,
        // Fix bad permissions for mounted stuff, then just stay alive
        command: ["sh", "-c", |||
            cp /update_badperms.sh /update &&
            chmod +x /update &&
            cp /ssh-secret/id_* ~/.ssh &&
            chmod 400 ~/.ssh/id_* &&
        ||| + commands,
        ],
        env_: {
            SOURCE_REPO: cfg.source.repo,
        },
        volumeMounts: [
            { name: "data", mountPath: '/data' },
            { name: "ssh", mountPath: '/ssh-secret' },
            { name: "config", subPath: "known_hosts", mountPath: '/root/.ssh/known_hosts' },
            { name: "config", subPath: "gitconfig", mountPath: '/root/.gitconfig' },
            { name: "config", subPath: "update.sh", mountPath: '/update_badperms.sh' },
        ],
    },

    deployment: ns.Contain(kube.Deployment(cfg.name)) {
        spec+: {
            replicas: cfg.replicas,
            minReadySeconds: 5,
            template+: {
                spec+: {
                    initContainers_: {
                        boostrap: makeManagementContainer("bootstrap", "/update")
                    },
                    containers_: {
                        default: kube.Container("default") {
                            image: cfg.images.nginx,
                            ports_: {
                                http: { containerPort: 8080 },
                            },
                            volumeMounts: [
                                {
                                    name: 'config',
                                    subPath: 'default.conf',
                                    mountPath: '/etc/nginx/conf.d/default.conf'
                                },
                                {
                                    name: 'data',
                                    mountPath: '/data'
                                },
                                {
                                    name: 'data',
                                    subPath: 'repo/' + cfg.source.staticSubpath,
                                    mountPath: '/usr/share/nginx/html'
                                },
                            ],
                        },
                    } + if cfg.enableManagement then {
                        // keep the container alive
                        management: makeManagementContainer("management", "sleep 2147483648")
                    } else {},
                    volumes_: volumes,
                },
            },
        },
    },

    // bleh, doesn't work, don't use
    updateJob:: ns.Contain(kube.Job(cfg.name + '-update')) {
        spec+: {
            ttlSecondsAfterFinished: 300,
            template+: {
                spec+: {
                    containers_: {
                        default: makeManagementContainer("update-job", "/update"),
                    },
                    volumes_: volumes,
                },
            },
        },
    },

    config: ns.Contain(kube.ConfigMap(cfg.name + '-config')) {
        data: {
            'default.conf': importstr 'default.conf',
            'known_hosts': importstr 'known_hosts',
            'gitconfig': importstr 'gitconfig',
            'update.sh': importstr 'update.sh',
        },
    },

    data: ns.Contain(kube.PersistentVolumeClaim(cfg.name)) {
        storage:: cfg.storage,
        storageClass:: cfg.storageClassName,
    },

    service: ns.Contain(kube.Service(cfg.name)) {
        target:: top.deployment,
    },

    ingress: ns.Contain(kube.SimpleIngress(cfg.name)) {
        hosts:: cfg.domains,
        target:: top.service,
    },
}
